////////////////////////////////////////////////////////////////////////////////
// File    : semantic.cpp
// Content : semantic behaviour implementation
// History : 18-Sep-2005 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2005  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* includes *******************************************************************/

#include "semantic.h"

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/Svgl.h>

#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGImageElement.hpp>

#include <string>
using namespace std;


/* Semantic behaviour implementation ******************************************/

Semantic::Semantic(svg::SVGElement *e, hsm::SVGLWindow *w) : 
	hsm::SVGLBehaviour(e, w), 
	scale(1.) {
		
	hsm::svgl::getAttribute(elem, "hsm-arg:scale", scale);
}


/* Thumb **********************************************************************/

Thumb::Thumb(svg::SVGElement *e, hsm::SVGLWindow *w) :
	hsm::SVGLBehaviour(e, w),
	hsm::SVGLTranslate(e, w),
	y(0.) {
	
	string targetId;
	hsm::svgl::getAttribute(elem, "hsm-arg:targetId", targetId);

	dom::Element *element = window->svgDocument->getElementById(unicode::String::createString(targetId.c_str()));
	content = dynamic_cast< svg::SVGImageElement * >(element);
}

void Thumb::translate(const hsm::Point &dp) {
	float dy = dp/hsm::Point::Y;
	y += dy;
	if((y < 0) || (y > 95)) {
		y -= dy;
		return;
	}
	
	hsm::Point d(2);
	d/hsm::Point::Y = dy;
	hsm::SVGLTranslate::translate(d);
	content->setY(-153 * y / 95);
}


// registration
const hsm::SVGLBehaviour::Factory::Register< Semantic > Semantic::registration("semantic");
const hsm::SVGLBehaviour::Factory::Register< Thumb > Thumb::registration("thumb");
const hsm::SVGLBehaviour::Factory::Register< NoTranslate > NoTranslate::registration("noTranslate");
