////////////////////////////////////////////////////////////////////////////////
// File    : semantic.h
// Content : semantic behaviour
// History : 18-Sep-2005 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2005  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#if !defined(SEMANTIC_H)
#define SEMANTIC_H


/* includes *******************************************************************/

#include <hsm/svgl/SVGLBehaviour.h>

namespace svg {
	class SVGImageElement;
}


/* Semantic behaviour *********************************************************/

class Semantic : public hsm::SVGLBehaviour, public virtual hsm::Protocol {
	static const hsm::SVGLBehaviour::Factory::Register< Semantic > registration;
	
	float scale;
	
public:
	Semantic(svg::SVGElement *e, hsm::SVGLWindow *w);

	inline float getScale() { return scale; }
};


/* Thumb **********************************************************************/

class Thumb : public hsm::SVGLTranslate {
private :
	static const hsm::SVGLBehaviour::Factory::Register< Thumb > registration;
	
	svg::SVGImageElement *content;
	float y;
public :
	Thumb(svg::SVGElement *e, hsm::SVGLWindow *w);
	virtual void translate(const hsm::Point &p);
};


/* NoTranslate ****************************************************************/

class NoTranslate : public hsm::SVGLBehaviour, public hsm::Translate {
private :
	static const hsm::SVGLBehaviour::Factory::Register< NoTranslate > registration;
	
public :
	NoTranslate(svg::SVGElement *e, hsm::SVGLWindow *w) :
		hsm::SVGLBehaviour(e, w) {}

	virtual void translate(const hsm::Point &p) {}
};


#endif // !defined(SEMANTIC_H)
