////////////////////////////////////////////////////////////////////////////////
// File    : sp.cpp
// Content : a simple program to test semantic pointing
// History : 11-Jul-2005 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
 Copyright (C) 2005  Renaud Blanch <rb.hsm@laposte.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <string>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/Device.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/ButtonPad.h>


#include <interactions.h>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "sp - a simple program to test semantic pointing." << endl << flush;
	atexit(quitHandler);

	// handling args
	string fileName;
	switch(argc) {
	case 1:
		fileName = "/Users/blanch/src/hsm/svg/sp.svg";
		break;
		
	case 2:
		fileName = argv[1];
		break;

	default:
		cerr << "usage : sp file.svg" << endl;
		return EXIT_FAILURE;
	}
	cout << "desktop file used: " << fileName << endl;
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("semantic pointing");
	window->setDocument(fileName);

	hsm::Device *pointer = hsm::repository::getDevice("Pointer");

	// creating instrument
	EnterLeaver::Hsm *enterLeaver = new EnterLeaver::Hsm(EnterLeaver::Data(window, *pointer/"position"));
	Actioner::Hsm *actioner = new Actioner::Hsm(Actioner::Data(window, *pointer/"position"/hsm::Point::cast, *pointer/"buttons"/hsm::ButtonPad::L));
	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));


	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	
	
	
	// clean-up
	delete translater;
	delete actioner;
	delete enterLeaver;
	delete window;
	
	
	return EXIT_SUCCESS;
}
