////////////////////////////////////////////////////////////////////////////////
// File    : 03-widgets.cpp
// Content : a simple program to test svgl widgets
// History : 24-Aug-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
 Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* includes *******************************************************************/

#include <interactions.h>

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/Device.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/ButtonPad.h>

#include <svgl/Parser.hpp>


/* button callback ************************************************************/

void buttonCb() { cout << "Hello " << flush; }
void crossCb() { cout << "world" << endl; }


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "widgets - a simple program to test svgl widgets." << endl << flush;
	atexit(quitHandler);
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("widgets");	
	hsm::Device *pointer = hsm::repository::getDevice("Pointer");
	
	// creating instruments
	EnterLeaver::Hsm *enterLeaver = new EnterLeaver::Hsm(EnterLeaver::Data(window, *pointer/"position"));
	Actioner::Hsm *actioner = new Actioner::Hsm(Actioner::Data(window, *pointer/"position"/hsm::Point::cast, *pointer/"buttons"/hsm::ButtonPad::L));
//	Crosser::Hsm *crosser = new Crosser::Hsm(Crosser::Data(window, *pointer/"position"));


	// registering button callback
	hsm::Functor::add("buttonCb", hsm::Functor::create(&buttonCb));
	hsm::Functor::add("crossCb", hsm::Functor::create(&crossCb));
	
	
	// creating the svg document
	svgl::Parser parser;
	std::stringstream strstr;
	strstr << 
		"<?xml version='1.0' encoding='iso-8859-1'?>"
		"<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 03December 1999//EN' 'http://www.w3.org/Graphics/SVG/SVG-19991203.dtd'>"
		"<svg xmlns:hsm='http://hsm.insitu.lri.fr/'"
		"     xmlns:hsm-arg='http://hsm.insitu.lri.fr/'"
		"     xmlns:xlink='http://www.w3.org/1999/xlink'"
		"     width='300' height='240'>"
		
		"	<defs>" 
		"		<linearGradient id='disarmed' gradientUnits='objectBoundingBox' x1='0' y1='0' x2='0' y2='1'>"
		"			<stop offset='0' style='stop-color:white' />"
		"			<stop offset='1' style='stop-color:lightgray' />"
		"		</linearGradient>"
		"		<linearGradient id='armed' gradientUnits='objectBoundingBox' x1='0' y1='0' x2='0' y2='1'>"
		"			<stop offset='0' style='stop-color:lightgray' />"
		"			<stop offset='1' style='stop-color:white' />"
		"		</linearGradient>"
		"		<symbol id='v'>"
		"			<path d='M -2 -1 h 4 l -2 2 z'/>"
		"		</symbol>"
		"		<symbol id='^'>"
		"			<path d='M -2 1 h 4 l -2 -2 z'/>"
		"		</symbol>"
		"	</defs>"

		// button definition
		"	<g hsm:behaviour='wimp/Button' hsm-arg:callbackId='buttonCb' style='font-size:15; font-family:Verdana; ' transform='translate(20,20)'>"
		"		<g>"
		"			<rect style='fill:url(#armed); stroke:gray' width='80' height='20' rx='4' ry='4'/>"
		"		</g>"
		"		<g>"
		"			<rect style='fill:url(#disarmed); stroke:gray' width='80' height='20' rx='4' ry='4'/>"
		"		</g>"
		"     <text x='15' y='15'>click it!</text>"
		"	</g>"
		
		// cross button definition
		"	<g hsm:behaviour='cross' hsm-arg:callbackId='crossCb' transform='translate(60,60)'>"
		"		<text style='font-size:15; font-family:Verdana; ' x='-29' y='13'>cross it!</text>"
		"		<g>"
		"			<use xlink:href='#v'/>"
		"			<use xlink:href='#^' y='18'/>"
		"		</g>"
		"		<rect style='fill:black; opacity:0' width='10' height='18' x='-10'/>"
		"		<rect style='fill:black; opacity:0' width='10' height='18' />"
		"	</g>"

		// tree view
		"	<defs>"
		"		<linearGradient id='bgd' gradientUnits='userSpaceOnUse' x1='0' y1='0' x2='40' y2='40'>"
		"			<stop offset='0' style='stop-color:lightgray' />"
		"			<stop offset='1' style='stop-color:white' />"
		"		</linearGradient>"
		"		<symbol id='closed'>"
		"			<path d='M 9 3 l 5 5 l -5 5 z' style='fill:gray' />"
		"		</symbol>"
		"		<symbol id='opened'>"
		"			<path d='M 6 6 l 5 5 l 5 -5 z' style='fill:gray' />"
		"		</symbol>"
		"		<clipPath id='clip-view'>"
		"			<rect width='200' height='1000'/>"
		"		</clipPath>"
		"	</defs>"
   
		"	<g transform='translate(120.5,20.5)' style='font-size:13; font-family:Andale Mono; clip-path:url(#clip-view);'>"
		"		<g hsm:behaviour='tree' hsm-arg:open='1'>"
		"			<rect width='1000' height='16' style='fill:url(#bgd)'/>"
		"			<use xlink:href='#opened' />"
		"			<text x='20' y='12.5'>~/test/</text>"
		
		"			<g transform='translate(16,16)'>"
		"				<g hsm:behaviour='node'>"
		"					<rect width='1000' height='16' style='fill:white; opacity:0;'/>"
		"					<text x='20' y='12.5'>README</text>"
		"				</g>"
		"				<g hsm:behaviour='node'>"
		"					<rect width='1000' height='16' style='fill:white; opacity:0;'/>"
		"					<text x='20' y='12.5'>INSTALL</text>"
		"				</g>"

		"				<g hsm:behaviour='tree' hsm-arg:open='0'>"
		"					<rect width='1000' height='16' style='fill:url(#bgd)'/>"
		"					<use xlink:href='#closed' />"
		"					<text x='20' y='12.5'>src/</text>"
		
		"					<g transform='translate(16,16)'>"
		"						<g hsm:behaviour='node'>"
		"							<rect width='1000' height='16' style='fill:white; opacity:0;'/>"
		"							<text x='20' y='12.5'>makefile</text>"
		"						</g>"
		"						<g hsm:behaviour='node'>"
		"							<rect width='1000' height='16' style='fill:white; opacity:0;'/>"
		"							<text x='20' y='12.5'>test.cpp</text>"
		"						</g>"
		"					</g>"
		"				</g>"

		"				<g hsm:behaviour='tree' hsm-arg:open='1'>"
		"					<rect width='1000' height='16' style='fill:url(#bgd)'/>"
		"					<use xlink:href='#closed' />"
		"					<text x='20' y='12.5'>doc/</text>"
		
		"					<g transform='translate(16,16)'>"
		"						<g hsm:behaviour='tree' hsm-arg:open='0'>"
		"							<rect width='1000' height='16' style='fill:url(#bgd)'/>"
		"							<use xlink:href='#closed' />"
		"							<text x='20' y='12.5'>img/</text>"

		"							<g transform='translate(16,16)'>"
		"								<g hsm:behaviour='node'>"
		"									<rect width='1000' height='16' style='fill:white; opacity:0;'/>"
		"									<text x='20' y='12.5'>test.jpg</text>"
		"								</g>"
		"								<g hsm:behaviour='node'>"
		"									<rect width='1000' height='16' style='fill:white; opacity:0;'/>"
		"									<text x='20' y='12.5'>test.png</text>"
		"								</g>"
		"							</g>"
		"						</g>"
		"						<g hsm:behaviour='node'>"
		"							<rect width='1000' height='16' style='fill:white; opacity:0;'/>"
		"							<text x='20' y='12.5'>index.html</text>"
		"						</g>"
		"						<g hsm:behaviour='node'>"
		"							<rect width='1000' height='16' style='fill:white; opacity:0;'/>"
		"							<text x='20' y='12.5'>toc.html</text>"
		"						</g>"
		"					</g>"
		"				</g>"
		"			</g>"
		"		</g>"
		"	</g>"

//		//	hierarchical menu definition
//		"	<g hsm:behaviour='menu' transform='translate(120,20)' style='fill:white; stroke:gray' >"
//		"		<g>"
//		"			<rect width='40' height='20'/>"
//		"        <text x='5' y='13' fill='black' >File</text>"
//		"		</g>"
//		"		<g>"
//		"			<g hsm:behaviour='menu' transform='translate(0,20)' style='fill:white'>"
//		"				<g>"
//		"					<rect width='40' height='20'/>"
//		"					<text x='5' y='13' fill='black' >Open</text>"
//		"				</g>"
//		"				<g>"
//		"					<g hsm:behaviour='menu' transform='translate(40,0)' style='fill:white'>"
//		"						<g>"
//		"							<rect width='40' height='20'/>"
//		"							<text x='5' y='13' fill='black' >open 1</text>"
//		"						</g>"
//		"					</g>"
//		"					<g hsm:behaviour='menu' transform='translate(40,20)' style='fill:white'>"
//		"						<g>"
//		"							<rect width='40' height='20'/>"
//		"							<text x='5' y='13' fill='black' >open 2</text>"
//		"						</g>"
//		"					</g>"
//		"				</g>"
//		"			</g>"
//		"			<g hsm:behaviour='menu' transform='translate(0,40)' style='fill:white'>"
//		"				<g>"
//		"					<rect width='40' height='20'/>"
//		"					<text x='5' y='13' fill='black' >Close</text>"
//		"				</g>"
//		"			</g>"
//		"		</g>"
//		"	</g>"
		
		"</svg>";
		
	svg::SVGDocument *document = parser.parseFromStream(&strstr);
	window->setDocument(document);
	

	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	
	
	
	// clean-up
	delete enterLeaver;
	delete actioner;
//	delete crosser;
	
	delete window;
	
	
	return EXIT_SUCCESS;
}
