////////////////////////////////////////////////////////////////////////////////
// File    : 04-bimanual.cpp
// Content : a simple program to test bimanual interactions
// History : 09-Jul-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
 Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/Device.h>
#include <hsm/system/event/EventLoop.h>

#include <svgl/Parser.hpp>

#include <interactions.h>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "bimanual - a program to test bimanual interaction." << endl << flush;
	atexit(quitHandler);

	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("bimanual");
	hsm::Device *pointer = hsm::repository::getDevice("Pointer");

	// palette
	hsm::Point palettePosition(2);
	palettePosition = 20, 20;
	palettePosition += *window/"position"/hsm::Point::cast;
	std::string path;
	char *cep = getenv("EXE_PATH");
	if(cep != 0) path = cep;
	hsm::SVGLCursor *toolglass = new hsm::SVGLCursor(palettePosition, window, path + "/toolglass.svg", window->svgFeedback);		

	// creating instruments
	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));
	EnterLeaver::Hsm *enterLeaver = new EnterLeaver::Hsm(EnterLeaver::Data(window, *pointer/"position"));
	Actioner::Hsm *actioner = new Actioner::Hsm(Actioner::Data(window, *pointer/"position"/hsm::Point::cast, *pointer/"buttons"/hsm::ButtonPad::L));
	TabletManager::Hsm *tabletManager = new TabletManager::Hsm(TabletManager::Data(window, toolglass));
	

	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	
	
	
	// clean-up
	delete translater;
	delete actioner;
	delete enterLeaver;
	delete tabletManager;
	
	delete window;
	
	
	return EXIT_SUCCESS;
}
