////////////////////////////////////////////////////////////////////////////////
// File    : 05-control.cpp
// Content : a simple program to test svgl widgets
// History : 24-Aug-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
 Copyright (C) 2005  Renaud Blanch <rb.hsm@laposte.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* includes *******************************************************************/

#include <interactions.h>

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/Device.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/ButtonPad.h>

#include <svgl/Parser.hpp>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "control - a program to test advanced interaction." << endl << flush;
	atexit(quitHandler);
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("control menu");	
	hsm::Device *pointer = hsm::repository::getDevice("Pointer");
	
	// creating instruments
	Controler::Hsm *controler = new Controler::Hsm(Controler::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"/hsm::Point::cast));

	// creating the svg document
	svgl::Parser parser;
	std::stringstream strstr;
	strstr << 
		"<?xml version='1.0' encoding='iso-8859-1'?>"
		"<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 03December 1999//EN' 'http://www.w3.org/Graphics/SVG/SVG-19991203.dtd'>"
		"<svg xmlns:hsm='http://hsm.insitu.lri.fr/'"
		"     xmlns:hsm-arg='http://hsm.insitu.lri.fr/'"
		"     xmlns:xlink='http://www.w3.org/1999/xlink'"
		"     width='300' height='240' style='stroke:none;'>"
		
		"	<g transform='translate(20,30)' hsm:behaviour='visual/Translatable'>"
		"		<rect style='fill:red;' width='80' height='60' />"
		"	</g>"

		"	<g transform='translate(200,50)' hsm:behaviour='visual/Zoomable'>"
		"		<rect style='fill:lightblue;' width='60' height='80' />"
		"	</g>"

		"	<g transform='translate(120,100)' hsm:behaviour='visual/PanAndZoomable'>"
		"		<rect style='fill:darkgreen;' width='70' height='70' />"
		"	</g>"
		
		"</svg>";
		
	svg::SVGDocument *document = parser.parseFromStream(&strstr);
	window->setDocument(document);
	

	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	
	
	
	// clean-up
	delete controler;
	
	delete window;
	
	
	return EXIT_SUCCESS;
}
