////////////////////////////////////////////////////////////////////////////////
// File    : ButtonPad.cpp
// Content : ButtonPad component
// History : 03-Dec-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <ButtonPad.h>

#include <sstream>
#include <cmath>


/* macros *********************************************************************/

#define MIN(n1, n2) ((n1 < n2)? n1 : n2)


namespace hsm {

/* ButtonPad class implementation *********************************************/

	const ButtonPad *ButtonPad::cast = 0;

	ButtonPad::ButtonPad(const Id &id, unsigned int d) : impl::Component< Composite >(id), dimension(d) {
		construct();
	}

	ButtonPad::ButtonPad(unsigned int d) : impl::Component< Composite >(), dimension(d) {
		construct();
	}

	ButtonPad::ButtonPad(const ButtonPad &b) : impl::Component< Composite >(b.componentId) {
		dimension = b.dimension;
		buttons.resize(dimension);		
		for(unsigned int i = 0; i < dimension; ++i) {
			std::stringstream buttonId;
			buttonId << i;
			buttons[i] = new Button(buttonId.str(), *b.buttons[i]);
			add(buttons[i]);
		}
	}

	ButtonPad::~ButtonPad() {
		for(unsigned int i = 0; i < dimension; ++i) {
			remove(buttons[i]);
			delete buttons[i];
		}
	}

	void ButtonPad::construct() {
		buttons.resize(dimension);
		for(unsigned int i = 0; i < dimension; ++i) {
			std::stringstream buttonId;
			buttonId << i;
			buttons[i] = new Button(buttonId.str());
			add(buttons[i]);
		}
	}

	void ButtonPad::process(const Event &event) {
		dispatch(Event(this, Event::CHANGE, (event.data.aBool? 1 : -1) * ((Button *)(event.sender))->getIndice()));
	}

	ButtonPad &ButtonPad::operator=(unsigned int bits) {
		for(unsigned int i = 0; i < dimension; ++i) {
			*buttons[i] = (((bits >> i) & 1) == 1);
		}
		return *this;
	}


	Button &operator/(const Component &buttonPad, ButtonPad::ButtonId i) {
		const ButtonPad *b = dynamic_cast< const ButtonPad * >(&buttonPad);
		return *b/i; // *(b->buttons[i]);
	}

	Button &operator/(const Component &buttonPad, char i) {
		const ButtonPad *b = dynamic_cast< const ButtonPad * >(&buttonPad);
		return *b/i; // *(b->buttons[i]);
	}
	
	std::ostream &operator<<(std::ostream &s, const ButtonPad &buttonPad) {
		s << "ButtonPad(";
		for(unsigned int i = 0; i < buttonPad.dimension; ++i) {
			s << *buttonPad.buttons[i];
		}
		return s << ")";
	}


} // namespace hsm


/* macros *********************************************************************/

#undef MIN
