////////////////////////////////////////////////////////////////////////////////
// File    : DragAndDrop.h
// Content : drag and drop protocol
// History : 06-AOU-2005 - [rb] creation
////////////////////////////////////////////////////////////////////////////////


#if !defined(DRAGANDDROP_H)
#define DRAGANDDROP_H


/* includes *******************************************************************/

#include <hsm/svgl/SVGLBehaviour.h>


/* forward declarations *******************************************************/

namespace svg {
	class SVGUseElement;
}


namespace hsm {

	class ThreadLoop;
	class Mutex;

	class Draggable;


/* DropZone class declaration *************************************************/

	class DropZone : public SVGLEnterLeave {
	public :
		DropZone(svg::SVGElement *e, SVGLWindow *w) : 
			SVGLBehaviour(e, w),
			SVGLEnterLeave(e, w) {
		}

		virtual DropZone *checkAccept(Draggable *) { return 0; }
		virtual void drop(Draggable *) = 0;
	};


/* Draggable declaration ******************************************************/

	class Draggable : public virtual SVGLBehaviour, public Translate {
	protected :
		svg::SVGUseElement *ghost;
		svg::SVGTranslateTransform *trans;

	private :
		DropZone *dropZone;

		Point origin;
		Point position;
		ThreadLoop *replacer;

		Mutex *singleInteraction;
	
		bool interruptedReplace;
	
	public :
		Draggable(svg::SVGElement *e, SVGLWindow *w);
		virtual ~Draggable();
	
		// inherited from Translate
		bool begin();
		void end();
		void translate(const Point &p);

	protected :
		virtual void setupGhost();

	private :
		void stepReplace();
		void removeGhost();
	};
	
	
} // namespace hsm


#endif // !defined(DRAGANDDROP_H)
