////////////////////////////////////////////////////////////////////////////////
// File    : ThreadLoop.cpp
// Content : implementation of ThreadLoop class
// History : 20-Dec-2001 - [rb] creation
//           05-Feb-2003 - [rb] added preamble and postamble
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2003  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <ThreadLoop.h>

#include <hsm/system/thread/Thread.h>
#include <hsm/util/Functor.h>


namespace hsm {

/* static data initialisation *************************************************/

	ThreadLoop::Map ThreadLoop::threadLoopMap;


/* ThreadLoop class implementation ********************************************/

	ThreadLoop::ThreadLoop(Functor *f, unsigned int ms /* = 0 */, void *d /* = 0 */, Functor *pre /* = 0 */, Functor *post /* = 0 */)
	 : preamble(pre), threadLoopFn(f), postamble(post), delay(ms), data(d), stopping(false), pausing(1) {
		threadId = os::create(this);
		threadLoopMap[threadId] = this;
	}
	
	ThreadLoop::~ThreadLoop() {
		stop();
		waitFor(this);
		threadLoopMap.erase(threadId);
	}

	void ThreadLoop::play() {
		pausing -= 1;
	}

	void ThreadLoop::pause() {
		pausing += 1;
	}

	void ThreadLoop::stop() {
		stopping = true;
	}

	void ThreadLoop::setDelay(unsigned int d) {
		delay = d;
	}

	ThreadLoop *ThreadLoop::get() {
		ThreadLoop *res = 0;
		Map::iterator i = threadLoopMap.find(os::self());
		if(i != threadLoopMap.end()){
			res = (*i).second;
		}
		return res;
	}

	void ThreadLoop::waitFor(const ThreadLoop *tl) {
		os::waitFor(tl->threadId);
	}

	void ThreadLoop::loop() {
		if(preamble != 0)
			(*preamble)();
			
		while(stopping == false) {
			if(pausing <= 0) {
				(*threadLoopFn)(data);
			}
			if(delay > 0)
				thread::sleep(delay);
		}

		if(postamble != 0)
			(*postamble)();
	}


} // namespace hsm
