////////////////////////////////////////////////////////////////////////////////
// File    : Device.h
// Content : Device component class
// History : 13-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Device.h
/// System device event handler.


#if !defined(HSM_DEVICE_H)
#define HSM_DEVICE_H


/* includes *******************************************************************/

#include <hsm/core/Composite.h>
#include <hsm/system/window/Window.h>


namespace hsm {

/* Device class definition ****************************************************/

	/// Implements a system device event handler
	class Device : public Composite {
	public :
		Window::os::EventSpec *eventSpec; ///< specification of the handled events
		unsigned int           nbEvents;  ///< # of handled events
		
	public :
		/// constructor
		Device(const Id &id) : Composite(id) {}
		
		/// returns the system event handler
		/// \return the system handler
		virtual Window::os::Handler *getHandler() const = 0;
		
		/// register Device with every current opened Window
		void registerDevice();
	};

	
} // namespace hsm

#endif // !defined(HSM_DEVICE_H)
