////////////////////////////////////////////////////////////////////////////////
// File    : GLWindow.WIN32.cpp
// Content : GLWindow::os struct implementation
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <GLWindow.h>


namespace hsm {

/* GLWindow::os struct implementation *****************************************/

	void GLWindow::os::createGlContext(GLWindow *glw) {
		PIXELFORMATDESCRIPTOR pfd = { sizeof(PIXELFORMATDESCRIPTOR),
		                              1,
		                              PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
		                              PFD_TYPE_RGBA,
		                              24,
		                              0, 0, 0, 0, 0, 0,
		                              0, 0, 0, 0, 0, 0, 0,
		                              32, 8,
		                              0,
		                              PFD_MAIN_PLANE,
		                              0,
		                              0, 0, 0 };

		HDC hDc = GetDC(glw->window);
		pfd.cColorBits = GetDeviceCaps(hDc, BITSPIXEL);
		SetPixelFormat(hDc, ChoosePixelFormat(hDc, &pfd), &pfd);

		glw->glWindowContext = wglCreateContext(hDc);
		glw->glCurrentContextContext = glw->glWindowContext;
	}

	void GLWindow::os::destroyGlContext(GLWindow *glw) {
		setFullScreen(glw, false);
		
		HDC hDc = GetDC(glw->window);
		wglMakeCurrent(hDc, 0);
		wglDeleteContext(glw->glCurrentContext);
		ReleaseDC(glw->window, hDc);	
	}
	
	void GLWindow::os::makeCurrent(GLWindow *glw) {
		wglMakeCurrent(GetDC(glw->window), glw->glCurrentContext);	
	}
	
	void GLWindow::os::swapBuffers(GLWindow *glw) {
		glFlush();
		SwapBuffers(GetDC(glw->window));	
	}

	void GLWindow::os::setVBLSync(GLWindow *glw, long vbl /* = 1 */) {}

	void GLWindow::os::setFullScreen(GLWindow *glw, bool full /* = true */) {}

	void GLWindow::os::reshape(GLWindow *glw) {}


} // namespace hsm
