////////////////////////////////////////////////////////////////////////////////
// File    : Pointer.WIN32.cpp
// Content : hsm system pointer component
// History : 18-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// ToDo ////////////////////////////////////////////////////////////////////////
// - allocate parts with physical caracteristics of the mouse
// - check if the mouse coordinates are in screen or window space (should be in
//   screen)


/* includes *******************************************************************/

#include <Pointer.h>

#include <hsm/util/Debug.h>
#include <hsm/component/Point.h>
#include <hsm/component/ButtonPad.h>

// to get mouse wheel
#include <windowsx.h>


namespace hsm {

/* Pointer instance ***********************************************************/

	const Pointer Pointer::instance;


/* Pointer implementation *****************************************************/

	LONG WINAPI Pointer::handler(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
		switch (iMessage) {
		case WM_MOUSEMOVE :
			*(instance.position) = GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam);
			break;

		case WM_MOUSEWHEEL :
			*(instance.wheel)/Point::W +=  (1. / WHEEL_DELTA) * ((short)HIWORD(wParam));
			break;

		case WM_LBUTTONDOWN :
			*(instance.buttons)/ButtonPad::L = true;
			break;
		case WM_LBUTTONUP :
			*(instance.buttons)/ButtonPad::L = false;
			break;

		case WM_MBUTTONDOWN :
			*(instance.buttons)/ButtonPad::M = true;
			break;
		case WM_MBUTTONUP :
			*(instance.buttons)/ButtonPad::M = false;
			break;

		case WM_RBUTTONDOWN :
			*(instance.buttons)/ButtonPad::R = true;
			break;
		case WM_RBUTTONUP :
			*(instance.buttons)/ButtonPad::R = false;
			break;

	 	default:
			return 0;
		}
		return 0;
	}


	Pointer::Pointer() : impl::Component< Device >("Pointer") {
		HSM_PRINT(1, "[Pointer::Pointer()]");

		nbEvents = 9;
		eventSpec = new Window::os::EventSpec[nbEvents];
		eventSpec[0] = WM_MOUSEMOVE;
		eventSpec[1] = WM_MOUSEWHEEL;
		eventSpec[2] = WM_LBUTTONDOWN;
		eventSpec[3] = WM_LBUTTONUP;
		eventSpec[4] = WM_MBUTTONDOWN;
		eventSpec[5] = WM_MBUTTONUP;
		eventSpec[6] = WM_RBUTTONDOWN;
		eventSpec[7] = WM_RBUTTONUP;
		eventSpec[8] = WM_MOVE;

		add(position = new Point("position", 2));
		add(wheel = new Point("wheel", 1));
		add(buttons = new ButtonPad("buttons", 3));

		hsm::repository::add(this);
		registerDevice();
	}

	Pointer::~Pointer() {
		HSM_PRINT(1, "[Pointer::~Pointer()]")
		hsm::repository::remove(this);
		delete[] eventSpec;
	}

	Window::os::Handler *Pointer::getHandler() const {
		return Pointer::handler;
	}


} // namespace hsm
