////////////////////////////////////////////////////////////////////////////////
// File    : test-Selector.cpp
// Content : a test for hsm::Selector
// History : 06-Feb-2003 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
using namespace std;

#include <hsm/system/select/Selector.h>
#include <hsm/util/Functor.h>
#include <hsm/core/Component.h>
#include <hsm/system/select/File.h>
#include <hsm/system/select/Alert.h>
#include <hsm/system/thread/ThreadLoop.h>
#include <hsm/system/time/Timer.h>
#include <hsm/system/event/EventLoop.h>


/* read & write ***************************************************************/

class Writer : public hsm::Functor {
	hsm::file::Descriptor fildes;
public :
	Writer(hsm::file::Descriptor fd) : fildes(fd) {}
	virtual void operator()(void *pData = 0) {
		static char c = 'a';
		write(fildes, &c, 1);
		if(++c > 'z') c = 'a';
	}
};

void listen(void *pData) {
	hsm::Event *event = reinterpret_cast< hsm::Event * >(pData);
	hsm::file::Descriptor fd = reinterpret_cast< hsm::file::Descriptor >(event->data.pVoid);
	hsm::Alert *a = dynamic_cast< hsm::Alert * >(hsm::Selectable::get(fd));
	if(a != 0) {
		// got an alert
		cout << "[Got an alert !!!]" << endl;
		a->reset();
	} else {
		// got something on a pipe 
		unsigned int nbyte = hsm::file::bytesToRead(fd);
		char *buf = new char[nbyte];
		read(fd, buf, nbyte);
		cout << "[";
		for(unsigned int i = 0; i < nbyte; cout << buf[i++]) {}
		cout << "]" << endl;
		delete[] buf;
	}
	
	delete event;
}


hsm::Alert *alert = 0;

void selectIt(void *s) {
	hsm::Selector *selector = reinterpret_cast< hsm::Selector * >(s);
	cout << "selector : " << selector << endl;
	selector->select();
	alert->reset();
}



/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "test-Selector - a program to test the hsm::Selector class." << endl << endl;
	atexit(quitHandler);

	alert = new hsm::Alert;
	
	hsm::Selector selector;
	hsm::Functor *listener = hsm::Functor::create(&listen);
	
	hsm::file::Descriptor p[2];
	pipe(p);
	

	cout << "starting writing in the pipe evry 1/5 s." << endl;
	hsm::ThreadLoop threadWriter(new Writer(p[1]), 200);
	threadWriter.play();

	cout << "starting sending alerts every second." << endl;
	hsm::ThreadLoop threadAlert(hsm::Functor::create(alert, &hsm::Alert::set), 1000);
	threadAlert.play();
	
	
	selector.add(*alert, hsm::Selector::R, listener);
	
	hsm::Timer wait(2000);
	hsm::Timer firstLoop(10000);
	bool res = true;
	while(res && !firstLoop.hasFired()) {
		wait.rearm();
		while(res && !wait.hasFired()) res = hsm::eventloop::step();
		if(!res) break;
		
		cout << endl << "adding reader on the pipe." << endl;
		selector.add(p[0], hsm::Selector::R, listener);

		wait.rearm();
		while(res && !wait.hasFired()) res = hsm::eventloop::step();

		cout << endl << "removing reader on the pipe." << endl;
		selector.remove(p[0], hsm::Selector::R, listener);
	}
	
	selector.remove(*alert, hsm::Selector::R, listener);
	threadWriter.stop();
	
	selector.add(*alert);
	
	hsm::ThreadLoop t1(hsm::Functor::create(&selectIt), 0, &selector);
	t1.play();
	
	while(true) {
		selector.select();
		alert->reset();
		cout << "loop" << endl;
	}
	
	
	delete listener;
	delete alert;
	
	return EXIT_SUCCESS;
}
